@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package Mojo::Content;
 
@@ -18,25 +18,11 @@ use Mojo::Headers;
 use constant MAX_MEMORY_SIZE => $ENV{MOJO_MAX_MEMORY_SIZE} || 10240;
 
 __PACKAGE__->attr(
-    [qw/buffer filter_buffer/] => (
-        chained => 1,
-        default => sub { Mojo::Buffer->new }
-    )
-);
-__PACKAGE__->attr([qw/body_cb filter builder_progress_cb/] => (chained => 1));
-__PACKAGE__->attr(
-    file => (
-        chained => 1,
-        default => sub { Mojo::File::Memory->new }
-    )
-);
-__PACKAGE__->attr(
-    headers => (
-        chained => 1,
-        default => sub { Mojo::Headers->new }
-    )
-);
-__PACKAGE__->attr(raw_header_length => (chained => 1, default => 0));
+    [qw/buffer filter_buffer/] => (default => sub { Mojo::Buffer->new }));
+__PACKAGE__->attr([qw/body_cb filter builder_progress_cb/]);
+__PACKAGE__->attr(file    => (default => sub { Mojo::File::Memory->new }));
+__PACKAGE__->attr(headers => (default => sub { Mojo::Headers->new }));
+__PACKAGE__->attr([qw/raw_header_length relaxed/] => (default => 0));
 
 sub build_body {
     my $self = shift;
@@ -111,6 +97,12 @@ sub get_header_chunk {
     return substr($copy, $offset, 4096);
 }
 
+sub has_leftovers {
+    my $self = shift;
+    return 1 if $self->buffer->length || $self->filter_buffer->length;
+    return 0;
+}
+
 sub header_length { return length shift->build_headers }
 
 sub is_chunked {
@@ -125,24 +117,25 @@ sub is_multipart {
     return $type =~ /multipart/i ? 1 : 0;
 }
 
+sub leftovers {
+    my $self = shift;
+
+    # Chunked leftovers are in the filter buffer, and so are those from a
+    # HEAD request
+    return $self->filter_buffer->to_string if $self->filter_buffer->length;
+
+    # Normal leftovers
+    return $self->buffer->to_string;
+}
+
 sub parse {
     my $self = shift;
 
     # Buffer
     $self->filter_buffer->add_chunk(join '', @_) if @_;
-    my $buffer = $self->filter_buffer;
-
-    # Parser started
-    if ($self->is_state('start')) {
-        my $length            = length($self->filter_buffer->{buffer});
-        my $raw_length        = $self->filter_buffer->raw_length;
-        my $raw_header_length = $raw_length - $length;
-        $self->raw_header_length($raw_header_length);
-        $self->state('headers');
-    }
 
     # Parse headers
-    $self->_parse_headers if $self->is_state('headers');
+    $self->parse_until_body;
 
     # Still parsing headers
     return $self if $self->is_state('headers');
@@ -177,15 +170,49 @@ sub parse {
         return Mojo::Content::MultiPart->new($self)->parse;
     }
 
-    # Parse body
-    $self->file->add_chunk($self->buffer->empty);
+    # Chunked body or relaxed content
+    if ($self->is_chunked || $self->relaxed) {
+        $self->file->add_chunk($self->buffer->empty);
+    }
 
-    # Done
-    unless ($self->is_chunked) {
+    # Normal body
+    else {
+
+        # Slurp
         my $length = $self->headers->content_length || 0;
+        my $need = $length - $self->file->length;
+        $self->file->add_chunk($self->buffer->remove($need)) if $need > 0;
+
+        # Done
         $self->done if $length <= $self->raw_body_length;
     }
 
+    # With leftovers, maybe pipelined
+    if ($self->is_done) {
+        $self->state('done_with_leftovers') if $self->has_leftovers;
+    }
+
+    return $self;
+}
+
+sub parse_until_body {
+    my $self = shift;
+
+    # Buffer
+    $self->filter_buffer->add_chunk(join '', @_) if @_;
+
+    # Parser started
+    if ($self->is_state('start')) {
+        my $length            = length($self->filter_buffer->{buffer});
+        my $raw_length        = $self->filter_buffer->raw_length;
+        my $raw_header_length = $raw_length - $length;
+        $self->raw_header_length($raw_header_length);
+        $self->state('headers');
+    }
+
+    # Parse headers
+    $self->_parse_headers if $self->is_state('headers');
+
     return $self;
 }
 
@@ -306,6 +333,11 @@ implements the following new ones.
 
     my $raw_body_length = $content->raw_body_length;
 
+=head2 C<relaxed>
+
+    my $relaxed = $content->relaxed;
+    $content    = $content->relaxed(1);
+
 =head1 METHODS
 
 L<Mojo::Content> inherits all methods from L<Mojo::Stateful> and implements
@@ -331,6 +363,10 @@ the following new ones.
 
     my $chunk = $content->get_header_chunk(13);
 
+=head2 C<has_leftovers>
+
+    my $leftovers = $content->has_leftovers;
+
 =head2 C<is_chunked>
 
     my $chunked = $content->is_chunked;
@@ -339,8 +375,18 @@ the following new ones.
 
     my $multipart = $content->is_multipart;
 
+=head2 C<leftovers>
+
+    my $bytes = $content->leftovers;
+
 =head2 C<parse>
 
     $content = $content->parse("Content-Length: 12\r\n\r\nHello World!");
 
+=head2 C<parse_until_body>
+
+    $content = $content->parse_until_body(
+        "Content-Length: 12\r\n\r\nHello World!"
+    );
+
 =cut
