@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package Mojo::File;
 
@@ -18,44 +18,46 @@ use Mojo::ByteStream;
 
 use constant TMPDIR => $ENV{MOJO_TMPDIR} || File::Spec->tmpdir;
 
-__PACKAGE__->attr('cleanup', chained => 1);
-__PACKAGE__->attr('handle',
-    chained => 1, 
-    default => sub {
-        my $self = shift;
-        my $handle = IO::File->new;
+__PACKAGE__->attr('cleanup');
+__PACKAGE__->attr(
+    handle => (
+        default => sub {
+            my $self   = shift;
+            my $handle = IO::File->new;
 
-        # Already got a file without handle
-        my $file = $self->path;
-        if ($file) {
+            # Already got a file without handle
+            my $file = $self->path;
+            if ($file) {
 
-            # New file
-            my $mode = '+>';
+                # New file
+                my $mode = '+>';
 
-            # File exists
-            $mode = '<' if -s $file;
+                # File exists
+                $mode = '<' if -s $file;
 
-            # Open
-            $handle->open("$mode $file")
-              or die qq/Can't open file "$file": $!/;
+                # Open
+                $handle->open("$mode $file")
+                  or die qq/Can't open file "$file": $!/;
+                return $handle;
+            }
+
+            # Generate temporary file
+            my $base = File::Spec->catfile(TMPDIR, 'mojo.tmp');
+            $file = $base;
+            while (-e $file) {
+                my $sum =
+                  Mojo::ByteStream->new(time . rand(999999999))->md5_sum;
+                $file = "$base.$sum";
+            }
+
+            $self->path($file);
+            $self->cleanup(1);
+
+            # Open for read/write access
+            $handle->open("+> $file") or die qq/Can't open file "$file": $!/;
             return $handle;
         }
-
-        # Generate temporary file
-        my $base = File::Spec->catfile(TMPDIR, 'mojo.tmp');
-        $file = $base;
-        while (-e $file) {
-            my $sum = Mojo::ByteStream->new(time . rand(999999999))->md5_sum;
-            $file = "$base.$sum";
-        }
-
-        $self->path($file);
-        $self->cleanup(1);
-
-        # Open for read/write access
-        $handle->open("+> $file") or die qq/Can't open file "$file": $!/;
-        return $handle;
-    }
+    )
 );
 
 sub DESTROY {
@@ -72,7 +74,7 @@ sub new {
 }
 
 sub add_chunk {
-    my $self  = shift;
+    my $self = shift;
     my $chunk = join '', @_ if @_;
 
     # Shortcut
@@ -258,4 +260,4 @@ Moves the uploaded file contents to the given path and returns the invocant.
 
 Returns the entire file content as a string.
 
-=cut
\ No newline at end of file
+=cut
