@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package Mojo::Message;
 
@@ -18,34 +18,42 @@ use Mojo::Parameters;
 use Mojo::Upload;
 use Mojo::URL;
 
-__PACKAGE__->attr('buffer',
-    chained => 1,
-    default => sub { Mojo::Buffer->new }
-);
-__PACKAGE__->attr([qw/
-    build_start_line_cb
-    parser_progress_cb
-/], chained => 1);
-__PACKAGE__->attr('content',
-    chained => 1,
-    default => sub { Mojo::Content->new }
-);
-__PACKAGE__->attr([qw/major_version minor_version/],
-    chained => 1,
-    default => 1
-);
+__PACKAGE__->attr(buffer => (default => sub { Mojo::Buffer->new }));
+__PACKAGE__->attr([qw/parser_progress_cb/]);
+__PACKAGE__->attr(content => (default => sub { Mojo::Content->new }));
+__PACKAGE__->attr([qw/major_version minor_version/] => (default => 1));
 
 # I'll keep it short and sweet. Family. Religion. Friendship.
 # These are the three demons you must slay if you wish to succeed in
 # business.
+sub at_least_version {
+    my ($self, $version) = @_;
+    my ($major, $minor) = split /\./, $version;
+
+    # Version is equal or newer
+    return 1 if $major > $self->major_version;
+    if ($major == $self->major_version) {
+        return 1 if $minor <= $self->minor_version;
+    }
+
+    # Version is older
+    return 0;
+}
+
 sub body {
     my ($self, $content) = @_;
 
     # Plain old content
     unless ($self->is_multipart) {
 
+        # Callback
+        if ($content && ref $content eq 'CODE') {
+            $self->body_cb($content);
+            return $content;
+        }
+
         # Get/Set content
-        if ($content) {
+        elsif ($content) {
             $self->content->file(Mojo::File::Memory->new);
             $self->content->file->add_chunk($content);
         }
@@ -56,6 +64,8 @@ sub body {
     return $self->content;
 }
 
+sub body_cb { shift->content->body_cb(@_) }
+
 sub body_length { shift->content->body_length }
 
 sub body_params {
@@ -68,8 +78,9 @@ sub body_params {
 
     # "x-application-urlencoded" and "application/x-www-form-urlencoded"
     my $content_type = $self->headers->content_type || '';
-    if ($content_type 
-      =~ /(?:x-application|application\/x-www-form)-urlencoded/i) {
+    if ($content_type
+        =~ /(?:x-application|application\/x-www-form)-urlencoded/i)
+    {
 
         # Parse
         my $raw = $self->content->file->slurp;
@@ -97,7 +108,7 @@ sub body_params {
 }
 
 sub build {
-    my $self = shift;
+    my $self    = shift;
     my $message = '';
 
     # Start line
@@ -118,8 +129,6 @@ sub build {
 # On top of a pile of money, with many beautiful women.
 sub build_body { shift->content->build_body(@_) }
 
-sub build_body_cb { shift->content->build_body_cb(@_) }
-
 sub build_headers {
     my $self = shift;
 
@@ -132,13 +141,11 @@ sub build_headers {
     return $self->content->build_headers;
 }
 
-sub build_headers_cb { shift->content->build_headers_cb(@_) }
-
 sub build_start_line {
     my $self = shift;
 
     my $startline = '';
-    my $offset = 0;
+    my $offset    = 0;
     while (1) {
         my $chunk = $self->get_start_line_chunk($offset);
 
@@ -185,14 +192,18 @@ sub cookie {
         $self->{_cookies} = $cookies;
     }
 
-    return $self->{_cookies}->{$name};
+    my @cookies =
+      ref $self->{_cookies}->{$name} eq 'ARRAY'
+      ? @{$self->{_cookies}->{$name}}
+      : ($self->{_cookies}->{$name});
+    return wantarray ? @cookies : $cookies[0];
 }
 
 sub fix_headers {
     my $self = shift;
 
     # Content-Length header is required in HTTP 1.0 messages
-    if ($self->is_version('1.0') && !$self->is_chunked) {
+    if ($self->at_least_version('1.0') && !$self->is_chunked) {
         $self->headers->content_length($self->body_length)
           unless $self->headers->content_length;
     }
@@ -206,7 +217,8 @@ sub get_header_chunk {
     my $self = shift;
 
     # Progress
-    $self->builder_progress_cb->($self) if $self->builder_progress_cb;
+    $self->builder_progress_cb->($self, 'headers', @_)
+      if $self->builder_progress_cb;
 
     # HTTP 0.9 has no headers
     return '' if $self->version eq '0.9';
@@ -221,17 +233,23 @@ sub get_start_line_chunk {
     my ($self, $offset) = @_;
 
     # Progress
-    $self->builder_progress_cb->($self) if $self->builder_progress_cb;
-
-    # Start line generator
-    return $self->build_start_line_cb->($self, $offset)
-      if $self->build_start_line_cb;
+    $self->builder_progress_cb->($self, 'start_line', $offset)
+      if $self->builder_progress_cb;
 
     my $copy = $self->_build_start_line;
     return substr($copy, $offset, 4096);
 }
 
-sub header_length { shift->content->header_length }
+sub has_leftovers { shift->content->has_leftovers }
+
+sub header_length {
+    my $self = shift;
+
+    # Fix headers
+    $self->fix_headers;
+
+    return $self->content->header_length;
+}
 
 sub headers { shift->content->headers(@_) }
 
@@ -239,19 +257,7 @@ sub is_chunked { shift->content->is_chunked }
 
 sub is_multipart { shift->content->is_multipart }
 
-sub is_version {
-    my ($self, $version) = @_;
-    my ($major, $minor) = split /\./, $version;
-
-    # Version is equal or newer
-    return 1 if $major > $self->major_version;
-    if ($major == $self->major_version) {
-        return 1 if $minor <= $self->minor_version;
-    }
-
-    # Version is older
-    return 0;
-}
+sub leftovers { shift->content->leftovers }
 
 sub param {
     my $self = shift;
@@ -266,20 +272,52 @@ sub parse {
     # Buffer
     $self->buffer->add_chunk(join '', @_) if @_;
 
+    return $self->_parse(0);
+}
+
+sub parse_until_body {
+    my $self = shift;
+
+    # Buffer
+    $self->buffer->add_chunk(join '', @_) if @_;
+
+    return $self->_parse(1);
+}
+
+sub _parse {
+    my $self = shift;
+    my $until_body = @_ ? shift : 0;
+
     # Progress
     $self->parser_progress_cb->($self) if $self->parser_progress_cb;
 
     # Content
-    if ($self->is_state(qw/content done/)) {
+    if ($self->is_state(qw/content done done_with_leftovers/)) {
         my $content = $self->content;
+
+        # HTTP 0.9 has no headers
         $content->state('body') if $self->version eq '0.9';
+
+        # Parse
         $content->filter_buffer($self->buffer);
-        $self->content($content->parse);
+
+        # Until body
+        if ($until_body) { $self->content($content->parse_until_body) }
+
+        # Whole message
+        else { $self->content($content->parse) }
+
+        # HTTP 0.9 has no defined length
+        $content->state('done') if $self->version eq '0.9';
     }
 
     # Done
     $self->done if $self->content->is_done;
 
+    # Done with leftovers, maybe pipelined
+    $self->state('done_with_leftovers')
+      if $self->content->is_state('done_with_leftovers');
+
     return $self;
 }
 
@@ -314,7 +352,11 @@ sub upload {
         $self->{_uploads} = $uploads;
     }
 
-    return $self->{_uploads}->{$name};
+    my @uploads =
+      ref $self->{_uploads}->{$name} eq 'ARRAY'
+      ? @{$self->{_uploads}->{$name}}
+      : ($self->{_uploads}->{$name});
+    return wantarray ? @uploads : $uploads[0];
 }
 
 sub uploads {
@@ -421,21 +463,12 @@ L<Mojo::Message> is a base class for HTTP messages.
 L<Mojo::Message> inherits all attributes from L<Mojo::Stateful> and
 implements the following new ones.
 
-=head2 C<body_length>
+=head2 C<body_cb>
 
-    my $body_length = $message->body_length;
-
-=head2 C<buffer>
-
-    my $buffer = $message->buffer;
-    $message   = $message->buffer(Mojo::Buffer->new);
-
-=head2 C<build_body_cb>
-
-    my $cb = $message->build_body_cb;
+    my $cb = $message->body_cb;
 
     $counter = 1;
-    $message = $message->build_body_cb(sub {
+    $message = $message->body_cb(sub {
         my $self  = shift;
         my $chunk = '';
         $chunk    = "hello world!" if $counter == 1;
@@ -444,23 +477,14 @@ implements the following new ones.
         return $chunk;
     });
 
-=head2 C<build_headers_cb>
+=head2 C<body_length>
 
-    my $cb = $message->build_headers_cb;
+    my $body_length = $message->body_length;
 
-    $message = $message->build_headers_cb(sub {
-        my $h = Mojo::Headers->new;
-        $h->content_type('text/plain');
-        return $h->to_string;
-    });
+=head2 C<buffer>
 
-=head2 C<build_start_line_cb>
-
-    my $cb = $message->build_start_line_cb;
-
-    $message = $content->build_start_line_cb(sub {
-        return "HTTP/1.1 200 OK\r\n\r\n";
-    });
+    my $buffer = $message->buffer;
+    $message   = $message->buffer(Mojo::Buffer->new);
 
 =head2 C<builder_progress_cb>
 
@@ -489,11 +513,17 @@ implements the following new ones.
     my $major_version = $message->major_version;
     $message          = $message->major_version(1);
 
+Returns the major version of the HTTP specification being followed.
+Defaults to 1.
+
 =head2 C<minor_version>
 
     my $minor_version = $message->minor_version;
     $message          = $message->minor_version(1);
 
+Returns the minor version of the HTTP specification being followed.
+Defaults to 1.
+
 =head2 C<parser_progress_cb>
 
     my $cb   = $message->parser_progress_cb;
@@ -520,10 +550,27 @@ implements the following new ones.
 L<Mojo::Message> inherits all methods from L<Mojo::Stateful> and implements
 the following new ones.
 
+=head2 C<at_least_version>
+
+    my $success = $message->at_least_version('1.1');
+
+Returns true if the HTTP version is greater than or equal to the version
+passed in.
+
 =head2 C<body>
 
     my $string = $message->body;
-    $message = $message->body('Hello!');
+    $message   = $message->body('Hello!');
+
+    $counter = 1;
+    $message = $message->body(sub {
+        my $self  = shift;
+        my $chunk = '';
+        $chunk    = "hello world!" if $counter == 1;
+        $chunk    = "hello world2!\n\n" if $counter == 2;
+        $counter++;
+        return $chunk;
+    });
 
 =head2 C<body_params>
 
@@ -537,26 +584,38 @@ Returns a L<Mojo::Parameters> object, containing POST parameters.
 
     my $string = $message->build;
 
+Returns the complete HTTP message.
+
 =head2 C<build_body>
 
     my $string = $message->build_body;
 
+Returns the HTTP message body.
+
 =head2 C<build_headers>
 
     my $string = $message->build_headers;
 
+Returns the HTTP message headers.
+
 =head2 C<build_start_line>
 
     my $string = $message->build_start_line;
 
+Returns the HTTP start line.
+
 =head2 C<cookie>
 
-    my $cookie = $message->cookie('foo');
+    my $cookie  = $message->cookie('foo');
+    my @cookies = $message->cookie('foo');
 
 =head2 C<fix_headers>
 
     $message = $message->fix_headers;
 
+Returns the invocant and makes sure all required headers for the currently
+followed HTTP version are set.
+
 =head2 C<get_body_chunk>
 
     my $string = $message->get_body_chunk($offset);
@@ -569,29 +628,39 @@ Returns a L<Mojo::Parameters> object, containing POST parameters.
 
     my $string = $message->get_start_line_chunk($offset);
 
+=head2 C<has_leftovers>
+
+    my $leftovers = $message->has_leftovers;
+
 =head2 C<is_chunked>
 
-    my $is_chunked = $message->is_chunked;
+    my $chunked = $message->is_chunked;
 
 =head2 C<is_multipart>
 
-    my $is_multipart = $message->is_multipart;
+    my $multipart = $message->is_multipart;
 
-=head2 C<is_version>
+=head2 C<leftovers>
 
-    my $is_version = $message->is_version('1.1);
+    my $bytes = $message->leftovers;
 
 =head2 C<param>
 
-    my $param = $message->param('foo');
+    my $param  = $message->param('foo');
+    my @params = $message->param('foo');
 
 =head2 C<parse>
 
     $message = $message->parse('HTTP/1.1 200 OK...');
 
+=head2 C<parse_until_body>
+
+    $message = $message->parse_until_body('HTTP/1.1 200 OK...');
+
 =head2 C<upload>
 
-    my $upload = $message->upload('foo');
+    my $upload  = $message->upload('foo');
+    my @uploads = $message->upload('foo');
 
 Returns a L<Mojo::Upload> object or a arrayref of L<Mojo::Upload> objects.
 
@@ -601,4 +670,4 @@ Returns a L<Mojo::Upload> object or a arrayref of L<Mojo::Upload> objects.
 
 Returns a arrayref of L<Mojo::Upload> objects.
 
-=cut
\ No newline at end of file
+=cut
