@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package MojoX::Dispatcher::Routes::Controller;
 
@@ -7,7 +7,7 @@ use warnings;
 
 use base 'Mojo::Base';
 
-__PACKAGE__->attr(ctx => (chained => 1));
+__PACKAGE__->attr('ctx');
 
 # If we don't go back there and make that event happen,
 # the entire universe will be destroyed...
