@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package MojoX::Routes;
 
@@ -13,13 +13,11 @@ use MojoX::Routes::Pattern;
 
 use constant DEBUG => $ENV{MOJOX_ROUTES_DEBUG} || 0;
 
-__PACKAGE__->attr([qw/block inline name/], chained => 1);
-__PACKAGE__->attr('children', chained => 1, default => sub { [] });
-__PACKAGE__->attr('parent', chained => 1, weak => 1);
-__PACKAGE__->attr('pattern',
-    chained => 1,
-    default => sub { MojoX::Routes::Pattern->new }
-);
+__PACKAGE__->attr([qw/block inline name/]);
+__PACKAGE__->attr(children => (default => sub { [] }));
+__PACKAGE__->attr(parent => (weak => 1));
+__PACKAGE__->attr(
+    pattern => (default => sub { MojoX::Routes::Pattern->new }));
 
 sub new {
     my $self = shift->SUPER::new();
@@ -32,6 +30,7 @@ sub bridge { return shift->route(@_)->inline(1) }
 sub is_endpoint {
     my $self = shift;
     return 0 if $self->inline;
+    return 1 if $self->block;
     return 0 if @{$self->children};
     return 1;
 }
@@ -49,33 +48,42 @@ sub match {
 
     # Path
     my $path = $match->path;
-    my $substring = $self->_shape(\$path);
-
-    # Debug
-    warn qq/"$substring" ("$path")\n/ if DEBUG;
 
     # Match
-    my $captures = $self->pattern->match($substring) || return undef;
+    my $captures = $self->pattern->shape_match(\$path);
 
     $match->path($path);
 
+    return undef unless $captures;
+
     # Merge captures
     $captures = {%{$match->captures}, %$captures};
     $match->captures($captures);
 
+    # Format
+    if ($self->is_endpoint && !$self->pattern->format) {
+        if ($path =~ /^\.([^\/]+)$/) {
+            $match->captures->{format} = $1;
+            $match->path('');
+        }
+    }
+    $match->captures->{format} = $self->pattern->format
+      if $self->pattern->format;
+
     # Update stack
-    if ($self->inline || $self->is_endpoint) {
+    if ($self->inline || ($self->is_endpoint && $match->is_path_empty)) {
         push @{$match->stack}, $captures;
     }
 
     # Waypoint match
-    if ($self->block && (!$path || $path eq '/')) {
+    if ($self->block && $match->is_path_empty) {
         push @{$match->stack}, $captures;
         $match->endpoint($self);
         return $self;
     }
 
     # Match children
+    my $snapshot = [@{$match->stack}];
     for my $child (@{$self->children}) {
 
         # Match
@@ -86,9 +94,12 @@ sub match {
 
         # Reset path
         $match->path($path);
+
+        # Reset stack
+        $match->stack($snapshot);
     }
 
-    $match->endpoint($self) if $self->is_endpoint;
+    $match->endpoint($self) if $self->is_endpoint && $match->is_path_empty;
 
     return $match;
 }
@@ -117,8 +128,6 @@ sub route {
     return $route;
 }
 
-sub segments { return shift->pattern->segments }
-
 sub to {
     my $self = shift;
 
@@ -145,6 +154,15 @@ sub url_for {
     # Path prefix
     my $path = $url->path->to_string;
     $path = $self->pattern->render($values) . $path;
+
+    # Make sure there is always a root
+    $path = '/' if !$path && !$self->parent;
+
+    # Format
+    if ((my $format = $values->{format}) && !$self->parent) {
+        $path .= ".$format" unless $path =~ /\.[^\/]+$/;
+    }
+
     $url->path->parse($path);
 
     $self->parent->url_for($url, $values) if $self->parent;
@@ -154,21 +172,6 @@ sub url_for {
 
 sub waypoint { return shift->route(@_)->block(1) }
 
-sub _shape {
-    my ($self, $pathref) = @_;
-
-    # Shortcut
-    return '' unless $self->segments;
-
-    my $substring = '';
-    for (1 .. $self->segments) {
-        $$pathref =~ s/^(\/?[^\/]*)//;
-        $substring .= $1;
-    }
-
-    return $substring;
-}
-
 1;
 __END__
 
@@ -218,10 +221,6 @@ L<MojoX::Routes> is a routes implementation.
     my $pattern = $routes->pattern;
     $routes     = $routes->pattern(MojoX::Routes::Pattern->new);
 
-=head2 C<segments>
-
-    my $segments = $routes->segments;
-
 =head1 METHODS
 
 L<MojoX::Routes> inherits all methods from L<Mojo::Base> and implements the
@@ -230,12 +229,12 @@ follwing the ones.
 =head2 C<new>
 
     my $routes = MojoX::Routes->new;
-    my $routes = MojoX::Routes->new('/:controller/:action');
+    my $routes = MojoX::Routes->new('/(controller)/(action)');
 
 =head2 C<bridge>
 
     my $bridge = $routes->bridge;
-    my $bridge = $routes->bridge('/:controller/:action');
+    my $bridge = $routes->bridge('/(controller)/(action)');
 
 =head2 C<to>
 
@@ -253,11 +252,11 @@ follwing the ones.
 
 =head2 C<parse>
 
-    $routes = $routes->parse('/:controller/:action');
+    $routes = $routes->parse('/(controller)/(action)');
 
 =head2 C<route>
 
-    my $route = $routes->route('/:c/:a', a => qr/\w+/);
+    my $route = $routes->route('/(c)/(a)', a => qr/\w+/);
 
 =head2 C<to_string>
 
@@ -270,6 +269,6 @@ follwing the ones.
 
 =head2 C<waypoint>
 
-    my $route = $routes->waypoint('/:c/:a', a => qr/\w+/);
+    my $route = $routes->waypoint('/(c)/(a)', a => qr/\w+/);
 
-=cut
\ No newline at end of file
+=cut
