@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package MojoX::Types;
 
@@ -7,17 +7,23 @@ use warnings;
 
 use base 'Mojo::Base';
 
-__PACKAGE__->attr('types', chained => 1, default => sub {{
-    css   => 'text/css',
-    ico   => 'image/x-icon',
-    gif   => 'image/gif',
-    html  => 'text/html',
-    jpg   => 'image/jpeg',
-    js    => 'application/x-javascript',
-    phtml => 'text/html',
-    png   => 'image/png',
-    txt   => 'text/plain'
-}});
+__PACKAGE__->attr(
+    types => (
+        default => sub {
+            return {
+                css   => 'text/css',
+                ico   => 'image/x-icon',
+                gif   => 'image/gif',
+                html  => 'text/html',
+                jpg   => 'image/jpeg',
+                js    => 'application/x-javascript',
+                phtml => 'text/html',
+                png   => 'image/png',
+                txt   => 'text/plain'
+            };
+        }
+    )
+);
 
 # Magic. Got it.
 sub type {
@@ -66,4 +72,4 @@ follwing the ones.
     my $type = $types->type('png');
     $types   = $types->type(png => 'image/png');
 
-=cut
\ No newline at end of file
+=cut
